#!/usr/bin/env python3
"""
Emergency Firewall Tool
Quickly deploys rules to protect a system under attack.
Parameters:
    - interface: Network interface to protect
    - mode: Protection mode (strict, adaptive, custom)
Requires: admin/root privileges
"""
import subprocess
import argparse
import logging
from datetime import datetime
import json
from pathlib import Path
import threading
import queue
import time
import psutil
import platform
import re

class EmergencyFirewall:
    def __init__(self, interface, mode='strict', whitelist=None):
        self.interface = interface
        self.mode = mode
        self.whitelist = whitelist or []
        self.is_windows = platform.system().lower() == 'windows'
        self.active_connections = set()
        self.blocked_ips = set()
        self.alert_queue = queue.Queue()
        self.stop_event = threading.Event()
        self.setup_logging()
        
    def setup_logging(self):
        """Set up logging configuration"""
        log_dir = Path("firewall_logs")
        log_dir.mkdir(exist_ok=True)
        
        logging.basicConfig(
            level=logging.INFO,
            format='%(asctime)s - %(levelname)s - %(message)s',
            handlers=[
                logging.FileHandler(log_dir / "emergency_firewall.log"),
                logging.StreamHandler()
            ]
        )
        
    def get_current_connections(self):
        """Get current network connections"""
        connections = set()
        for conn in psutil.net_connections():
            if conn.raddr and conn.raddr.ip:
                connections.add(conn.raddr.ip)
        return connections
        
    def is_suspicious(self, ip):
        """Check if IP shows suspicious behavior"""
        if ip in self.whitelist:
            return False
            
        # Check for common attack patterns
        connection_count = len([x for x in psutil.net_connections()
                            if x.raddr and x.raddr.ip == ip])
        
        return connection_count > 10  # Threshold for suspicious connections
        
    def create_windows_rules(self):
        """Create Windows Firewall rules"""
        try:
            # Block all inbound traffic except whitelisted
            subprocess.run([
                'netsh', 'advfirewall', 'set', 'allprofiles',
                'state', 'on'
            ], check=True)
            
            # Create strict inbound rule
            subprocess.run([
                'netsh', 'advfirewall', 'firewall', 'add', 'rule',
                'name="Emergency Lockdown"',
                'dir=in', 'action=block',
                'enable=yes'
            ], check=True)
            
            # Allow whitelisted IPs
            for ip in self.whitelist:
                subprocess.run([
                    'netsh', 'advfirewall', 'firewall', 'add', 'rule',
                    f'name="Emergency Whitelist {ip}"',
                    'dir=in', 'action=allow',
                    f'remoteip={ip}'
                ], check=True)
                
            logging.info("Windows Firewall rules created successfully")
            
        except subprocess.CalledProcessError as e:
            logging.error(f"Error creating Windows Firewall rules: {str(e)}")
            
    def create_linux_rules(self):
        """Create Linux iptables rules"""
        try:
            # Flush existing rules
            subprocess.run(['iptables', '-F'], check=True)
            
            # Set default policies
            subprocess.run(['iptables', '-P', 'INPUT', 'DROP'], check=True)
            subprocess.run(['iptables', '-P', 'FORWARD', 'DROP'], check=True)
            subprocess.run(['iptables', '-P', 'OUTPUT', 'ACCEPT'], check=True)
            
            # Allow loopback
            subprocess.run(['iptables', '-A', 'INPUT', '-i', 'lo', '-j', 'ACCEPT'], check=True)
            
            # Allow established connections
            subprocess.run([
                'iptables', '-A', 'INPUT', '-m', 'state',
                '--state', 'ESTABLISHED,RELATED', '-j', 'ACCEPT'
            ], check=True)
            
            # Allow whitelisted IPs
            for ip in self.whitelist:
                subprocess.run([
                    'iptables', '-A', 'INPUT', '-s', ip, '-j', 'ACCEPT'
                ], check=True)
                
            logging.info("Linux iptables rules created successfully")
            
        except subprocess.CalledProcessError as e:
            logging.error(f"Error creating iptables rules: {str(e)}")
            
    def block_ip(self, ip):
        """Block specific IP address"""
        if ip in self.blocked_ips:
            return
            
        try:
            if self.is_windows:
                subprocess.run([
                    'netsh', 'advfirewall', 'firewall', 'add', 'rule',
                    f'name="Emergency Block {ip}"',
                    'dir=in', 'action=block',
                    f'remoteip={ip}'
                ], check=True)
            else:
                subprocess.run([
                    'iptables', '-A', 'INPUT', '-s', ip, '-j', 'DROP'
                ], check=True)
                
            self.blocked_ips.add(ip)
            logging.info(f"Blocked suspicious IP: {ip}")
            
        except subprocess.CalledProcessError as e:
            logging.error(f"Error blocking IP {ip}: {str(e)}")
            
    def monitor_connections(self):
        """Monitor network connections for suspicious activity"""
        while not self.stop_event.is_set():
            current_connections = self.get_current_connections()
            
            # Check for new connections
            new_connections = current_connections - self.active_connections
            for ip in new_connections:
                if self.is_suspicious(ip):
                    self.alert_queue.put({
                        'type': 'suspicious_connection',
                        'ip': ip,
                        'timestamp': datetime.now().isoformat()
                    })
                    self.block_ip(ip)
                    
            self.active_connections = current_connections
            time.sleep(1)
            
    def handle_alerts(self):
        """Handle and log security alerts"""
        while not self.stop_event.is_set():
            try:
                alert = self.alert_queue.get(timeout=1)
                if alert['type'] == 'suspicious_connection':
                    logging.warning(
                        f"Suspicious connection detected from {alert['ip']}"
                    )
            except queue.Empty:
                continue
                
    def deploy_emergency_rules(self):
        """Deploy emergency firewall rules"""
        logging.info("Deploying emergency firewall rules")
        logging.info(f"Mode: {self.mode}")
        logging.info(f"Interface: {self.interface}")
        
        if self.whitelist:
            logging.info(f"Whitelist: {', '.join(self.whitelist)}")
            
        # Create initial rules
        if self.is_windows:
            self.create_windows_rules()
        else:
            self.create_linux_rules()
            
        # Start monitoring threads
        monitor_thread = threading.Thread(target=self.monitor_connections)
        alert_thread = threading.Thread(target=self.handle_alerts)
        
        monitor_thread.start()
        alert_thread.start()
        
        try:
            while True:
                # Print status
                print(f"\rActive connections: {len(self.active_connections)}, "
                      f"Blocked IPs: {len(self.blocked_ips)}", end="")
                time.sleep(1)
                
        except KeyboardInterrupt:
            logging.info("\nStopping emergency firewall...")
            self.stop_event.set()
            monitor_thread.join()
            alert_thread.join()
            
            # Save final state
            self.save_state()
            
    def save_state(self):
        """Save firewall state and statistics"""
        state = {
            'timestamp': datetime.now().isoformat(),
            'mode': self.mode,
            'interface': self.interface,
            'blocked_ips': list(self.blocked_ips),
            'whitelist': self.whitelist,
            'active_connections': len(self.active_connections)
        }
        
        state_file = Path("firewall_logs") / f"state_{datetime.now().strftime('%Y%m%d_%H%M%S')}.json"
        with open(state_file, 'w') as f:
            json.dump(state, f, indent=2)
            
        logging.info(f"State saved to {state_file}")
        
    def restore_system(self):
        """Restore system to original state"""
        logging.info("Restoring system firewall configuration")
        
        try:
            if self.is_windows:
                # Remove emergency rules
                subprocess.run([
                    'netsh', 'advfirewall', 'firewall', 'delete', 'rule',
                    'name="Emergency Lockdown"'
                ], check=True)
                
                for ip in self.blocked_ips:
                    subprocess.run([
                        'netsh', 'advfirewall', 'firewall', 'delete', 'rule',
                        f'name="Emergency Block {ip}"'
                    ], check=True)
                    
            else:
                # Flush iptables
                subprocess.run(['iptables', '-F'], check=True)
                subprocess.run(['iptables', '-P', 'INPUT', 'ACCEPT'], check=True)
                subprocess.run(['iptables', '-P', 'FORWARD', 'ACCEPT'], check=True)
                
            logging.info("System restored to original state")
            
        except subprocess.CalledProcessError as e:
            logging.error(f"Error restoring system: {str(e)}")

if __name__ == "__main__":
    parser = argparse.ArgumentParser(description="Emergency Firewall Tool")
    parser.add_argument("interface", help="Network interface to protect")
    parser.add_argument("--mode", choices=['strict', 'adaptive', 'custom'],
                      default='strict', help="Protection mode")
    parser.add_argument("--whitelist", help="Comma-separated list of IPs to whitelist")
    args = parser.parse_args()
    
    whitelist = args.whitelist.split(',') if args.whitelist else []
    
    firewall = EmergencyFirewall(args.interface, args.mode, whitelist)
    try:
        firewall.deploy_emergency_rules()
    except KeyboardInterrupt:
        firewall.restore_system()
